﻿using System;
using System.AddIn.Contract;
using System.AddIn.Pipeline;
using System.IO;
using System.Windows;
using System.Windows.Forms.Integration;

namespace HostingLib
{
    public static class CrossDomainHost
    {
        const string SlotName = "Slot.01F1C93D-90F9-4D8A-86E4-44BE215E6CAE";

        public static FrameworkElement CreateHost()
        {
            var location = typeof(CrossDomainHost).Assembly.Location;
            var thisDir = Path.GetDirectoryName(location);
            var pluginsDir = Path.Combine(thisDir, "Plugins");

            var setup = new AppDomainSetup
            {
                ApplicationBase = thisDir,
                PrivateBinPath = pluginsDir,
                ConfigurationFile = Path.Combine(pluginsDir, "WinFormApp.exe.config"),
                AppDomainInitializer = InitDomain
            };

            var domain = AppDomain.CreateDomain("PluginsDomain", null, setup);
            var contract = (INativeHandleContract)domain.GetData(SlotName);
            var control = FrameworkElementAdapters.ContractToViewAdapter(contract);
            return control;
        }

        private static void InitDomain(string[] args)
        {
            var winFormControl = (System.Windows.Forms.Control)AppDomain.CurrentDomain.CreateInstanceAndUnwrap("WinFormApp", "WinFormApp.MainControl");
            var host = new WindowsFormsHost() { Child = winFormControl };
            var contract = FrameworkElementAdapters.ViewToContractAdapter(host);
            AppDomain.CurrentDomain.SetData(SlotName, contract);
        }
    }
}
